# SPDX-FileCopyrightText: 2017-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from argparse import Namespace
from datetime import date, timedelta

from gvm.protocols.gmp import Gmp
from terminaltables import AsciiTable


def check_args(args: Namespace) -> None:
    len_args = len(args.script) - 1
    if len_args < 2:
        message = """
        This script will display all vulnerabilities from the hosts of the
        reports in a given month!
        It needs two parameters after the script name.
        First one is the month and second one is the year.
        Both parameters are plain numbers, so no text.
        Explicitly made for GOS 4.X.

        1. <month>  -- month of the monthly report
        2. <year>   -- year of the monthly report

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
    ssh --hostname <gsm> scripts/monthly-report2.gmp.py 05 2019
        """
        print(message)
        sys.exit()


def print_reports(gmp: Gmp, from_date: date, to_date: date) -> None:
    host_filter = (
        f"rows=-1 and modified>{from_date.isoformat()} "
        f"and modified<{to_date.isoformat()}"
    )

    hosts_xml = gmp.get_hosts(filter_string=host_filter)

    sum_high = 0
    sum_medium = 0
    sum_low = 0
    table_data = [["Hostname", "IP", "Bericht", "high", "medium", "low"]]

    for host in hosts_xml.xpath("asset"):
        ip = host.xpath("name/text()")[0]

        hostnames = host.xpath(
            'identifiers/identifier/name[text()="hostname"]/../value/text()'
        )

        if len(hostnames) == 0:
            continue

        hostname = hostnames[0]

        results = gmp.get_results(
            details=False, filter=f"host={ip} and severity>0.0"
        )

        low = int(results.xpath('count(//result/threat[text()="Low"])'))
        sum_low += low

        medium = int(results.xpath('count(//result/threat[text()="Medium"])'))
        sum_medium += medium

        high = int(results.xpath('count(//result/threat[text()="High"])'))
        sum_high += high

        best_os_cpe_report_id = host.xpath(
            'host/detail/name[text()="best_os_cpe"]/../source/@id'
        )[0]

        table_data.append(
            [hostname, ip, best_os_cpe_report_id, high, medium, low]
        )

    table = AsciiTable(table_data)
    print(f"{table.table}\n")
    print(
        f"Summary of results from {from_date.isoformat()} "
        f"to {to_date.isoformat()}"
    )
    print(f"High: {int(sum_high)}")
    print(f"Medium: {int(sum_medium)}")
    print(f"Low: {int(sum_low)}\n\n")


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable

    check_args(args)

    month = int(args.script[1])
    year = int(args.script[2])
    from_date = date(year, month, 1)
    to_date = from_date + timedelta(days=31)
    # To have the first day in month
    to_date = to_date.replace(day=1)

    print_reports(gmp, from_date, to_date)


if __name__ == "__gmp__":
    main(gmp, args)



# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see<http://www.gnu.org/licenses/>


import unittest
from argparse import Namespace
from pathlib import Path
from unittest.mock import patch

from lxml import etree

from . import GmpMockFactory, load_script

CWD = Path(__file__).absolute().parent


class SendSchedulesTestCase(unittest.TestCase):
    def setUp(self):
        self.send_schedules = load_script(
            (CWD.parent.parent / "scripts"), "send-schedules"
        )

    @patch("gvm.protocols.latest.Gmp", new_callable=GmpMockFactory)
    def test_sent_schedule(self, mock_gmp: GmpMockFactory):
        schedule_xml_path = CWD / "example_schedules.xml"
        schedule_xml_str = schedule_xml_path.read_text(encoding="utf-8")

        mock_gmp.mock_responses(
            "create_schedule",
            [
                '<create_schedule_response status="201" status_text="OK,'
                'resource created" id="75be149a-0877-40f9-97c0-dfea31311e35"/>',
                '<create_schedule_response status="201" status_text="OK,'
                'resource created" id="42da6616-f32d-47b4-8d6b-2e4553c42ee7"/>',
            ],
        )

        schedule = etree.XML(schedule_xml_str)

        self.send_schedules.parse_send_xml_tree(mock_gmp.gmp_protocol, schedule)

    @patch("gvm.protocols.latest.Gmp", new_callable=GmpMockFactory)
    def test_args(self, mock_gmp: GmpMockFactory):
        args = Namespace(script=["foo"])
        with self.assertRaises(SystemExit):
            self.send_schedules.check_args(gmp=mock_gmp, args=args)

        args2 = Namespace(script=["foo", "bar", "baz"])

        with self.assertRaises(SystemExit):
            self.send_schedules.check_args(gmp=mock_gmp, args=args2)



# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
Main module of gvm-tools.
"""


def get_version() -> str:
    """Returns the version of gvm-tools as a string in `PEP440`_ compliant
    format.

    Returns:
        str: Current version of gvm-tools

    .. _PEP440:
       https://www.python.org/dev/peps/pep-0440
    """
    # pylint: disable=import-outside-toplevel
    from .__version__ import __version__

    return __version__



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

# Run with gvm-script --gmp-username admin-user --gmp-password password socket start-scans-from-csv.gmp.py startscans.csv

import csv
import sys
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from pathlib import Path

from gvm.errors import GvmResponseError
from gvm.protocols.gmp import Gmp
from gvmtools.helper import error_and_exit

HELP_TEXT = "This script pulls task names from a csv file and starts the tasks listed in every row. \n"


def check_args(args):
    len_args = len(args.script) - 1
    if len_args != 2:
        message = """
        This script pulls tasks from a csv file and creates a \
task for each row in the csv file.
        One parameter after the script name is required.

        1. <tasks_csvfile>  -- csv file containing names and secrets required for scan tasks

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
ssh --hostname <gsm> scripts/stop_tasks_from_csv.gmp.py \
<tasks-csvfile>
        """
        print(message)
        sys.exit()


def parse_args(args: Namespace) -> Namespace:  # pylint: disable=unused-argument
    """Parsing args ..."""

    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
        description=HELP_TEXT,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    parser.add_argument(
        "task_file",
        type=str,
        help=("CSV File containing tasks"),
    )
    script_args, _ = parser.parse_known_args(args)
    return script_args


def task_id(
    gmp: Gmp,
    task_name: str,
):
    response_xml = gmp.get_tasks(
        filter_string="rows=-1, status=Running "
        "or status=Requested "
        "or status=Queued "
        "and name=" + task_name
    )
    tasks_xml = response_xml.xpath("task")
    task_id = ""

    for task in tasks_xml:
        task_id = task.get("id")
    return task_id


def stop_tasks(
    gmp: Gmp,
    task_file: Path,
):
    try:
        numbertasks = 0
        with open(task_file, encoding="utf-8") as csvFile:
            content = csv.reader(csvFile, delimiter=",")  # read the data
            try:
                for row in content:  # loop through each row
                    if len(row) == 0:
                        continue
                    task_stop = task_id(gmp, row[0])
                    if task_stop:
                        numbertasks = numbertasks + 1
                        print(
                            f"Stopping task name: {row[0]} with uuid: {task_stop} ..."
                        )
                        status_text = gmp.stop_task(task_stop).xpath(
                            "@status_text"
                        )[0]
                        print(status_text)
                    else:
                        print(
                            "Task "
                            + row[0]
                            + " is either in status Stopped, Stop Requested, or does not exist on this system.\n"
                        )
            except GvmResponseError as gvmerr:
                print(f"{gvmerr=}, task: {task_stop}")
                pass
        csvFile.close()  # close the csv file

    except IOError as e:
        error_and_exit(f"Failed to read task_file: {str(e)} (exit)")

    if len(row) == 0:
        error_and_exit("tasks file is empty (exit)")

    return numbertasks


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable
    if args.script:
        args = args.script[1:]

    parsed_args = parse_args(args=args)

    print("Stopping tasks.\n")

    numbertasks = stop_tasks(
        gmp,
        parsed_args.task_file,
    )

    numbertasks = str(numbertasks)
    print("   \n [" + numbertasks + "] task(s)/scan(s) stopped!\n")


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# Loosely based on the create-filter-from-host-list
#
# Run with gvm-script --gmp-username admin-user --gmp-password password socket create-filters-from-csv.gmp.py hostname-server filters.csv

import csv
import sys
import time
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from pathlib import Path

from gvm.errors import GvmResponseError
from gvm.protocols.gmp import Gmp
from gvmtools.helper import error_and_exit

HELP_TEXT = (
    "This script pulls filtername, hostnames/IP addresses, and credentials "
    "from a csv file and creates a filter for each row. \n\n"
    "csv file to contain name of filter, ips, and up to 4 credentials previously created \n"
    "name,ip-addresses,credential1,credential2,credential3,credential4"
)


def check_args(args):
    len_args = len(args.script) - 1
    if len_args != 1:
        message = """
        This script pulls filter names from a csv file and creates a filter \
for each row.
        One parameter after the script name is required.

        1. <filters_csvfile>  -- text file containing filtername and hostnames or IP-addresses

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
ssh --hostname <gsm> scripts/create_filters_from_csv.gmp <filters_csvfile>
        """
        print(message)
        sys.exit()


def parse_args(args: Namespace) -> Namespace:  # pylint: disable=unused-argument
    """Parsing args ..."""

    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
        description=HELP_TEXT,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    parser.add_argument(
        "filters_csv_file",
        type=str,
        help=("File containing host names / IPs"),
    )

    script_args, _ = parser.parse_known_args(args)
    return script_args


def filter_id(
    gmp: Gmp,
    filter_name: str,
):
    response_xml = gmp.get_filters(filter_string="rows=-1, name=" + filter_name)
    filters_xml = response_xml.xpath("filter")
    filter_id = ""

    for filter in filters_xml:
        filter_id = filter.get("id")
    return filter_id


def create_filters(
    gmp: Gmp,
    filter_csv_file: Path,
):
    try:
        numberfilters = 0
        with open(filter_csv_file, encoding="utf-8") as csvFile:
            content = csv.reader(csvFile, delimiter=",")  # read the data
            for row in content:  # loop through each row
                if len(row) == 0:
                    continue
                filterType = row[0]
                filterName = row[1]
                filterDescription = row[2]
                filterTerm = row[3]
                filterNameFull = (
                    filterName + ":" + filterDescription + ":" + filterType
                )
                comment = f"Created: {time.strftime('%Y/%m/%d-%H:%M:%S')}"
                if filterType == "FAIL!":
                    print(filterType.upper())
                elif filterType.upper() == "ALERT":
                    resource_type = gmp.types.FilterType.ALERT
                elif filterType.upper() == "ASSET":
                    resource_type = gmp.types.FilterType.ASSET
                elif filterType.upper() == "CONFIG":
                    resource_type = gmp.types.FilterType.SCAN_CONFIG
                elif filterType.upper() == "CREDENTIAL":
                    resource_type = gmp.types.FilterType.CREDENTIAL
                elif filterType.upper() == "HOST":
                    resource_type = gmp.types.FilterType.HOST
                elif filterType.upper() == "SECINFO":
                    resource_type = gmp.types.FilterType.ALL_SECINFO
                elif filterType.upper() == "NOTE":
                    resource_type = gmp.types.FilterType.NOTE
                elif filterType.upper() == "OS":
                    resource_type = gmp.types.FilterType.OPERATING_SYSTEM
                elif filterType.upper() == "OVERRIDE":
                    resource_type = gmp.types.FilterType.OVERRIDE
                elif filterType.upper() == "PERMISSION":
                    resource_type = gmp.types.FilterType.PERMISSION
                elif filterType.upper() == "PORT_LIST":
                    resource_type = gmp.types.FilterType.PORT_LIST
                elif filterType.upper() == "REPORT":
                    resource_type = gmp.types.FilterType.REPORT
                elif filterType.upper() == "REPORT_FORMAT":
                    resource_type = gmp.types.FilterType.REPORT_FORMAT
                elif filterType.upper() == "RESULT":
                    resource_type = gmp.types.FilterType.RESULT
                elif filterType.upper() == "ROLE":
                    resource_type = gmp.types.FilterType.ROLE
                elif filterType.upper() == "SCHEDULE":
                    resource_type = gmp.types.FilterType.SCHEDULE
                elif filterType.upper() == "TAG":
                    resource_type = gmp.types.FilterType.TAG
                elif filterType.upper() == "TARGET":
                    resource_type = gmp.types.FilterType.TARGET
                elif filterType.upper() == "TASK":
                    resource_type = gmp.types.FilterType.TASK
                elif filterType.upper() == "TICKET":
                    resource_type = gmp.types.FilterType.TICKET
                elif filterType.upper() == "TLS_CERTIFICATE":
                    resource_type = gmp.types.FilterType.TLS_CERTIFICATE
                elif filterType.upper() == "USER":
                    resource_type = gmp.types.FilterType.USER
                elif filterType.upper() == "VULNERABILITY":
                    resource_type = gmp.types.FilterType.VULNERABILITY
                else:
                    print(
                        "FilterType: " + filterType.upper() + " Not supported"
                    )
                try:
                    if filter_id(gmp, filterNameFull):
                        print(
                            f"Filter: {filterNameFull} exist, not creating..."
                        )
                        continue

                    print("Creating filter: " + filterNameFull)
                    gmp.create_filter(
                        name=filterNameFull,
                        comment=comment,
                        filter_type=resource_type,
                        term=filterTerm,
                    )
                    numberfilters = numberfilters + 1
                except GvmResponseError as gvmerr:
                    print(f"{gvmerr=}, name: {filterNameFull}")
                    pass
        csvFile.close()  # close the csv file
    except IOError as e:
        error_and_exit(f"Failed to read filter_csv_file: {str(e)} (exit)")

    if len(row) == 0:
        error_and_exit("filter file is empty (exit)")

    return numberfilters


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable
    if args.script:
        args = args.script[1:]

    parsed_args = parse_args(args=args)

    print("Creating filters.\n")

    numberfilters = create_filters(
        gmp,
        parsed_args.filters_csv_file,
    )

    numberfilters = str(numberfilters)
    print("    [" + numberfilters + "] filter(s) created!\n")


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from datetime import date
from typing import List, Tuple
from uuid import UUID

from gvm.errors import GvmError
from gvm.protocols.gmp import Gmp
from gvmtools.helper import error_and_exit, generate_uuid
from lxml import etree as e

HELP_TEXT = (
    "This script creates a consolidated report and imports it to the GSM. "
    "You are able to set a time period. Within this period the last report"
    "of all tasks will be consolidated. You can additionally filter the "
    "tasks by one or more tags and the results with a filter id or filter "
    "term.\n"
    " Usable with gvm-script (gvm-tools). Help: gvm-script -h"
)


def parse_tags(tags: List[str]) -> List[str]:
    """Parsing and validating the given tags

    tags (List): A list containing tags:
                 name, tag-id, name=value

    Returns a list containing tag="name", tag_id="id" ...
    """
    filter_tags = []
    for tag in tags:
        try:
            UUID(tag, version=4)
            filter_tags.append(f'tag_id="{tag}"')
        except ValueError:
            filter_tags.append(f'tag="{tag}"')

    return filter_tags


def parse_period(period: List[str]) -> Tuple[date, date]:
    """Parsing and validating the given time period

    period (List): A list with two entries containing
                   dates in the format yyyy/mm/dd

    Returns two date-objects containing the passed dates
    """
    try:
        s_year, s_month, s_day = map(int, period[0].split("/"))
    except ValueError as exc:
        error_and_exit(
            f"Start date [{period[0]}] is not a "
            f"correct date format:\n{exc.args[0]}."
        )
    try:
        e_year, e_month, e_day = map(int, period[1].split("/"))
    except ValueError as exc:
        error_and_exit(
            f"End date [{period[1]}] is not "
            f"a correct date format:\n{exc.args[0]}."
        )

    try:
        period_start = date(s_year, s_month, s_day)
    except ValueError as exc:
        error_and_exit(f"Start date: {exc.args[0]}")

    try:
        period_end = date(e_year, e_month, e_day)
    except ValueError as exc:
        error_and_exit(f"End date: {exc.args[0]}")

    if period_end < period_start:
        error_and_exit("The start date seems to after the end date.")

    return period_start, period_end


def parse_args(args: Namespace) -> Namespace:  # pylint: disable=unused-argument
    """Parsing args ..."""

    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
        description=HELP_TEXT,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    parser.add_argument(
        "+p",
        "++period",
        nargs=2,
        type=str,
        required=True,
        dest="period",
        help=(
            "Choose a time period that is filtering the tasks.\n"
            "Use the date format YYYY/MM/DD."
        ),
    )

    container_args = parser.add_mutually_exclusive_group()

    container_args.add_argument(
        "++container-id",
        type=str,
        dest="container_id",
        help=(
            "Add the consolidated report to the container task"
            " with the given id instead of creating a new one."
        ),
    )

    container_args.add_argument(
        "++new-container-name",
        type=str,
        dest="new_container_name",
        help=(
            "Create a new container task with the given name instead"
            " of using an automatically generated name."
        ),
    )

    parser.add_argument(
        "+t",
        "++tags",
        nargs="+",
        type=str,
        dest="tags",
        help=(
            "Filter the tasks by given tag(s).\n"
            "If you pass more than on tag, they will be concatenated with "
            or "\n"
            "You can pass tag names, tag ids or tag name=value to this argument"
        ),
    )

    filter_args = parser.add_mutually_exclusive_group()

    filter_args.add_argument(
        "++filter-terms",
        nargs="+",
        type=str,
        dest="filter_term",
        help="Filter the results by given filter terms.",
    )

    filter_args.add_argument(
        "++filter-id",
        type=str,
        dest="filter_id",
        help="Filter the results by given filter id.",
    )

    script_args, _ = parser.parse_known_args()
    return script_args


def generate_task_filter(
    period_start: date, period_end: date, tags: List[str]
) -> str:
    """Generate the tasks filter

    period_start: the start date
    period_end: the end date
    tags: list of tags for the filter

    Returns an task filter string
    """
    task_filter = "rows=-1 "

    # last is for the timestamp of the last report in that task
    # created is for the timestamp of when the task has been created
    # Note: the "first" argument for tasks is currently not working
    period_filter = (
        f"last>{period_start.isoformat()} "
        f"and created<{period_end.isoformat()}"
    )
    filter_parts = []
    if tags:
        for tag in tags:
            filter_parts.append(f"{period_filter} and {tag}")

        tags_filter = " or ".join(filter_parts)
        task_filter += tags_filter
    else:
        task_filter += period_filter

    return task_filter


def get_last_report_in_time_period(
    gmp: Gmp,
    task_filter: str,
    period_start: date,
    period_end: date,
) -> List[str]:
    """Get the last reports from the tasks in the given time period
    Therefore all tasks, that match the filter within the time period
    will be looked up
    Afterwards the reports from that tasks will be searched for the
    last report in the timeperiod by sorting them reverse by creation
    date

    gmp: the GMP object
    task_filter: task filter string

    """
    print(
        f"Filtering the task with the filter term [{task_filter}]\n"
        f"Looking for the last report before {period_end.isoformat()}, "
        f"but after {period_start.isoformat()}."
    )

    tasks_xml = gmp.get_tasks(filter_string=task_filter)
    reports = []
    for task_id in tasks_xml.xpath("task/@id"):
        # sort-reverse for getting the latest report ...
        reports_xml = gmp.get_reports(
            filter_string=(
                f"rows=1 task_id={task_id} and "
                f"created<{period_end.isoformat()} and "
                f"created>{period_start.isoformat()} sort-reverse=created"
            )
        )
        # should always be max 1 report
        reports_id = reports_xml.xpath("report/@id")
        if reports_id:
            reports.append(reports_id[0])
        else:
            print(f"Failed to get report for task {task_id}", file=sys.stderr)
            sys.exit(1)
    return reports


def combine_reports(
    gmp: Gmp, reports: List[str], filter_term: str, filter_id: str
) -> e.Element:
    """Combining the filtered ports, results and hosts of the given
    report ids into one new report.

    gmp: the GMP object
    reports (List): List of report_ids
    filter_term (str): the result filter string
    """

    new_uuid = generate_uuid()
    combined_report = e.Element(
        "report",
        {
            "id": new_uuid,
            "format_id": "d5da9f67-8551-4e51-807b-b6a873d70e34",
            "extension": "xml",
            "content_type": "text/xml",
        },
    )
    report_elem = e.Element("report", {"id": new_uuid})

    ports_elem = e.Element("ports", {"start": "1", "max": "-1"})
    results_elem = e.Element("results", {"start": "1", "max": "-1"})
    combined_report.append(report_elem)
    report_elem.append(ports_elem)
    report_elem.append(results_elem)

    for report in reports:
        try:
            if filter_id:
                current_report = gmp.get_report(
                    report,
                    filter_id=filter_id,
                    details=True,
                    ignore_pagination=True,
                ).find("report")
            else:
                current_report = gmp.get_report(
                    report,
                    filter_string=filter_term,
                    details=True,
                    ignore_pagination=True,
                ).find("report")
        except GvmError:
            print(f"Could not find the report [{report}]")
        for port in current_report.xpath("report/ports/port"):
            ports_elem.append(port)
        for result in current_report.xpath("report/results/result"):
            results_elem.append(result)
        for host in current_report.xpath("report/host"):
            report_elem.append(host)

    return combined_report


def get_container_name(gmp: Gmp, container_id: str):
    """
    Gets the name of a task by id and checks if it is a container

    gmp: the GMP object
    container_id: Id of the task to check
    """
    res = gmp.get_task(container_id)

    task_name = res.xpath("//task/name")[0].text
    task_target_id = res.xpath("//task/target/@id")[0]

    if task_target_id != "":
        error_and_exit(f"Task [{container_id}] is not a container")

    return task_name


def send_report(
    gmp: Gmp,
    combined_report: e.Element,
    period_start: date,
    period_end: date,
    container_id: str,
    new_container_name: str,
) -> str:
    """Creating a container task and sending the combined report to the GSM

    gmp: the GMP object
    combined_report: the combined report xml object
    period_start: the start date
    period_end: the end date
    """

    task_id = None
    task_name = None

    if container_id:
        task_name = get_container_name(gmp, container_id)
        task_id = container_id
        print(
            "Adding consolidated report to existing container task"
            f" [{task_name}] with UUID [{task_id}]"
        )
    else:
        if new_container_name:
            task_name = new_container_name
        else:
            task_name = f"Consolidated Report [{period_start} - {period_end}]"

        res = gmp.create_container_task(
            name=task_name, comment="Created with gvm-tools."
        )

        task_id = res.xpath("//@id")[0]
        print(
            "Adding consolidated report to new container task"
            f" [{task_name}] with UUID [{task_id}]"
        )

    combined_report = e.tostring(combined_report)

    res = gmp.import_report(combined_report, task_id=task_id, in_assets=True)

    return res.xpath("//@id")[0]


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable

    parsed_args = parse_args(args=args)

    period_start, period_end = parse_period(period=parsed_args.period)

    print(
        "Combining last reports from tasks within the "
        f"time period [{period_start}, {period_end}]"
    )

    # Generate Task Filter
    filter_tags = None
    if parsed_args.tags:
        filter_tags = parse_tags(tags=parsed_args.tags)

    task_filter = generate_task_filter(
        period_start=period_start,
        period_end=period_end,
        tags=filter_tags,
    )

    # Find reports
    reports = get_last_report_in_time_period(
        gmp=gmp,
        task_filter=task_filter,
        period_start=period_start,
        period_end=period_end,
    )

    print(f"Combining {len(reports)} found reports.")

    filter_term = ""
    if parsed_args.filter_term:
        filter_term = " ".join(parsed_args.filter_term)
        print(
            "Filtering the results by the "
            f"following filter term [{filter_term}]"
        )
    elif parsed_args.filter_id:
        try:
            filter_xml = gmp.get_filter(filter_id=parsed_args.filter_id).find(
                "filter"
            )
            filter_term = filter_xml.find("term").text
            print(
                "Filtering the results by the following filter term "
                f"[{filter_term}]"
            )
        except GvmError:
            print(
                "Filter with the ID [{parsed_args.filter_id}] is not existing."
            )
    else:
        print("No results filter given.")

    # Combine the reports
    combined_report = combine_reports(
        gmp=gmp,
        reports=reports,
        filter_term=filter_term,
        filter_id=parsed_args.filter_id,
    )

    # Import the generated report to GSM
    report = send_report(
        gmp=gmp,
        combined_report=combined_report,
        period_start=period_start,
        period_end=period_end,
        container_id=parsed_args.container_id,
        new_container_name=parsed_args.new_container_name,
    )

    print(f"Successfully imported new consolidated report [{report}]")


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import Namespace

from gvm.protocols.gmp import Gmp
from gvmtools.helper import Table

# from gvm.xml import pretty_print


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=unused-argument

    response_xml = gmp.get_feeds()
    feeds_xml = response_xml.xpath("feed")
    heading = ["#", "Name", "Version", "Type", "Status"]
    rows = []
    numberRows = 0
    #    pretty_print(feeds_xml)

    print("Listing feeds and their status.\n")

    for feed in feeds_xml:
        # Count number of reports
        numberRows = numberRows + 1
        # Cast/convert to text to show in list
        rowNumber = str(numberRows)
        name = "".join(feed.xpath("name/text()"))
        version = "".join(feed.xpath("version/text()"))
        feed_type = "".join(feed.xpath("type/text()"))
        status = "".join(feed.xpath("currently_syncing/timestamp/text()"))
        if not status:
            status = "Up-to-date..."
        else:
            status = "Update in progress..."

        rows.append([rowNumber, name, version, feed_type, status])

    print(Table(heading=heading, rows=rows))


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2018 inovex GmbH
# SPDX-FileCopyrightText: 2019-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from argparse import Namespace
from typing import List

from gvm.protocols.gmp import Gmp


def check_args(args):
    len_args = len(args.script) - 1
    message = """
        This script makes an alert scan
        It needs two parameters after the script name.

        1. <sender_email>     -- E-Mail of the sender
        2. <receiver_email>   -- E-Mail of the receiver

                Example:
            $ gvm-script --gmp-username name --gmp-password pass \
ssh --hostname <gsm> scripts/start-multiple-alert-scan.gmp.py \
<sender_email> <receiver_email>
    """
    if len_args != 2:
        print(message)
        sys.exit()


# returns a list containing all port_list names
def get_port_list_names(gmp) -> List[str]:
    res = gmp.get_port_lists()
    port_names_list = [""]
    for name in res.findall("port_list/name"):
        port_names_list.append(str(name.text))
    return port_names_list


def get_scan_config(gmp, debug=False):
    # get all configs of the openvas instance
    res = gmp.get_scan_configs()

    # configurable template
    template = "fast"

    # match the config abbreviation to accepted config names
    config_list = [
        "Full and fast",
        "Full and fast ultimate",
        "Full and very deep",
        "Full and very deep ultimate",
        "System Discovery",
    ]
    template_abbreviation_mapper = {
        "fast": config_list[0],
        "fast-ulti": config_list[1],
        "deep": config_list[2],
        "deep-ulti": config_list[3],
        "discovery": config_list[4],
    }
    config_id = "-"
    for conf in res.xpath("config"):
        cid = conf.xpath("@id")[0]
        name = conf.xpath("name/text()")[0]

        # get the config id of the desired template
        if template_abbreviation_mapper.get(template, "-") == name:
            config_id = cid
            if debug:
                print(f"{name}: {config_id}")
            break
    # check for existence of the desired config
    if config_id == "-":
        print(
            f"error: could not recognize template '{template}'\n"
            f"valid template names are: {config_list}\n"
        )
        exit()

    return config_id


def get_target(gmp, debug=False):
    # find a targetName
    targets = gmp.get_targets()

    counter = 0
    exists = True

    # iterate over existing targets and find a vacant targetName
    while exists:
        exists = False
        target_name = f"targetName{str(counter)}"
        for target in targets.xpath("target"):
            name = target.xpath("name/text()")[0]
            if name == target_name:
                exists = True
                break
        counter += 1

    if debug:
        print(f"target name: {target_name}")

    # iterate over existing port lists and find a vacant name
    new_port_list_name = "portlistName"
    counter = 0

    while True:
        portlist_name = f"{new_port_list_name}{str(counter)}"
        if portlist_name not in get_port_list_names(gmp):
            break
        counter += 1

    # configurable port string
    port_string = "T:80-80"
    # create port list
    portlist = gmp.create_port_list(portlist_name, port_string)
    portlist_id = portlist.xpath("@id")[0]
    if debug:
        print(f"Portlist-name:\t{str(portlist_name)}")
        print(f"Portlist-id:\t{str(portlist_id)}")

    # configurable hosts
    hosts = ["localhost"]

    # integrate port list id into create_target
    res = gmp.create_target(
        name=target_name, hosts=hosts, port_list_id=portlist_id
    )
    return res.xpath("@id")[0]


def get_alerts(gmp, sender_email, recipient_email, debug=False) -> List[str]:
    # configurable alert name
    alert_name = recipient_email

    # create alert if necessary
    alert_object = gmp.get_alerts(filter=f"name={alert_name}")
    alert_id = None
    alert = alert_object.xpath("alert")
    if len(alert) == 0:
        gmp.create_alert(
            alert_name,
            event=gmp.types.AlertEvent.TASK_RUN_STATUS_CHANGED,
            event_data={"status": "Done"},
            condition=gmp.types.AlertCondition.ALWAYS,
            method=gmp.types.AlertMethod.EMAIL,
            method_data={
                """Task '$n': $e

After the event $e,
the following condition was met: $c

This email escalation is configured to attach report format '$r'.
Full details and other report formats are available on the scan engine.

$t

Note:
This email was sent to you as a configured security scan escalation.
Please contact your local system administrator if you think you
should not have received it.
""": "message",
                "2": "notice",
                sender_email: "from_address",
                "[OpenVAS-Manager] Task": "subject",
                "c402cc3e-b531-11e1-9163-406186ea4fc5": "notice_attach_format",
                recipient_email: "to_address",
            },
        )
        alert_object = gmp.get_alerts(filter=f"name={recipient_email}")
        alert = alert_object.xpath("alert")
        alert_id = alert[0].get("id", "no id found")
    else:
        alert_id = alert[0].get("id", "no id found")
        if debug:
            print(f"alert_id: {str(alert_id)}")

    # second configurable alert name
    alert_name2 = f"{recipient_email}-2"

    # create second alert if necessary
    alert_object2 = gmp.get_alerts(filter=f"name={alert_name2}")
    alert_id2 = None
    alert2 = alert_object2.xpath("alert")
    if len(alert2) == 0:
        gmp.create_alert(
            alert_name2,
            event=gmp.types.AlertEvent.TASK_RUN_STATUS_CHANGED,
            event_data={"status": "Done"},
            condition=gmp.types.AlertCondition.ALWAYS,
            method=gmp.types.AlertMethod.EMAIL,
            method_data={
                """Task '$n': $e

After the event $e,
the following condition was met: $c

This email escalation is configured to attach report format '$r'.
Full details and other report formats are available on the scan engine.

$t

Note:
This email was sent to you as a configured security scan escalation.
Please contact your local system administrator if you think you
should not have received it.
""": "message",
                "2": "notice",
                sender_email: "from_address",
                "[OpenVAS-Manager] Task": "subject",
                recipient_email: "to_address",
            },
        )
        alert_object2 = gmp.get_alerts(filter=f"name={recipient_email}")
        alert2 = alert_object2.xpath("alert")
        alert_id2 = alert2[0].get("id", "no id found")
    else:
        alert_id2 = alert2[0].get("id", "no id found")
        if debug:
            print(f"alert_id2: {str(alert_id2)}")

    return [alert_id, alert_id2]


def get_scanner(gmp):
    res = gmp.get_scanners()
    scanner_ids = res.xpath("scanner/@id")
    return scanner_ids[1]  # default scanner


def create_and_start_task(
    gmp, config_id, target_id, scanner_id, alerts, debug=False
):
    # Create the task
    tasks = gmp.get_tasks(filter="name~ScanDoneMultipleAlert")
    task_name = f"ScanDoneMultipleAlert{len(tasks.xpath('tasks/@id'))}"
    task_comment = "test comment"
    res = gmp.create_task(
        name=task_name,
        config_id=config_id,
        target_id=target_id,
        scanner_id=scanner_id,
        alert_ids=alerts,
        comment=task_comment,
    )
    # Start the task
    task_id = res.xpath("@id")[0]
    gmp.start_task(task_id)

    print(f"Task started: {task_name}")

    if debug:
        # Stop the task (for performance reasons)
        gmp.stop_task(task_id)
        print("Task stopped")


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable

    check_args(args)

    sender_email = args.script[1]
    recipient_email = args.script[2]

    config_id = get_scan_config(gmp)
    target_id = get_target(gmp)
    alerts = get_alerts(gmp, sender_email, recipient_email)
    scanner_id = get_scanner(gmp)

    create_and_start_task(gmp, config_id, target_id, scanner_id, alerts)

    print("\nScript finished\n")


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from argparse import Namespace

from gvm.protocols.gmp import Gmp


def check_args(args):
    len_args = len(args.script) - 1

    if len_args != 3:
        message = """
        This script modifies the timezone and/or icalendar of all schedules
        in a filter selection.

        <filter>    -- the filter text used to filter the schedules.
        <timezone>  -- the new timezone to set or empty to keep existing one.
        <icalendar> -- the new icalendar to set or empty to keep existing one.

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \\
                ssh --hostname <gsm> scripts/bulk-modify-schedules.gmp.py \\
                <filter> <timezone> <icalendar>
        """
        print(message)
        sys.exit()


def bulk_modify_schedules(gmp, filter_term, new_timezone, new_icalendar):
    get_response = gmp.get_schedules(filter=filter_term)
    schedules = get_response.findall("schedule")

    for schedule in schedules:
        uuid = schedule.attrib["id"]
        name = schedule.find("name").text
        comment = schedule.find("comment").text

        if new_timezone:
            timezone = new_timezone
        else:
            timezone = schedule.find("timezone").text

        if new_icalendar:
            icalendar = new_icalendar
        else:
            icalendar = schedule.find("icalendar").text

        print(f"- Modifying {name} ({uuid})")

        gmp.modify_schedule(
            uuid,
            name=name,
            comment=comment,
            timezone=timezone,
            icalendar=icalendar,
        )


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=unused-argument

    check_args(args)

    filter_term = args.script[1]
    new_timezone = args.script[2]
    new_icalendar = args.script[3]

    bulk_modify_schedules(gmp, filter_term, new_timezone, new_icalendar)


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import Namespace

from gvm.protocols.gmp import Gmp
from gvmtools.helper import Table


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=unused-argument

    response_xml = gmp.get_report_formats(details=True, filter_string="rows=-1")
    report_formats_xml = response_xml.xpath("report_format")
    heading = ["#", "Name", "Id", "Summary"]
    rows = []
    numberRows = 0

    print("Listing report formats.\n")

    for report_format in report_formats_xml:
        # Count number of reports
        numberRows = numberRows + 1
        # Cast/convert to text to show in list
        rowNumber = str(numberRows)
        name = "".join(report_format.xpath("name/text()"))
        report_format_id = report_format.get("id")
        report_format_summary = "".join(report_format.xpath("summary/text()"))

        rows.append([rowNumber, name, report_format_id, report_format_summary])

    print(Table(heading=heading, rows=rows))


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2017-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
from argparse import Namespace

from gvm.protocols.gmp import Gmp


def check_args(args):
    len_args = len(args.script) - 1
    if len_args != 2:
        message = """
        This script creates a new task with specific host and nvt!
        It needs two parameters after the script name.
        First one is the oid of the nvt and the second one is the
        chosen scan target.

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
ssh --hostname <gsm> scripts/start-nvt-scan.gmp.py \
    1.3.6.1.4.1.25623.1.0.106223 localhost
        """
        print(message)
        sys.exit()


def get_scan_config(gmp, nvt_oid):
    # Choose from existing config, which to copy or create new config
    res = gmp.get_scan_configs()

    config_ids = res.xpath("config/@id")

    for i, conf in enumerate(res.xpath("config")):
        config_id = conf.xpath("@id")[0]
        name = conf.xpath("name/text()")[0]
        print(f"\n({i}) {name}: ({config_id})")

    while True:
        chosen_config = input(
            "\nChoose your config or create new one"
            f"[0-{len(config_ids) - 1} | n]: "
        )

        if chosen_config == "n":
            chosen_copy_config = int(
                input(f"Which config to copy from? [0-{len(config_ids) - 1}]: ")
            )
            config_name = input("Enter new Name for config: ")

            copy_id = config_ids[chosen_copy_config]

            res = gmp.clone_scan_config(copy_id)

            config_id = res.xpath("@id")[0]

            # Modify the config with an nvt oid
            if len(nvt_oid) == 0:
                nvt_oid = input("NVT OID: ")

            nvt = gmp.get_scan_config_nvt(nvt_oid=nvt_oid)
            family = nvt.xpath("nvt/family/text()")[0]

            gmp.modify_scan_config(
                config_id,
                "nvt_selection",
                name=config_name,
                nvt_oids=[nvt_oid],
                family=family,
            )

            # This nvts must be present to work
            family = "Port scanners"
            nvts = [
                "1.3.6.1.4.1.25623.1.0.14259",
                "1.3.6.1.4.1.25623.1.0.100315",
            ]

            gmp.modify_scan_config(
                config_id, "nvt_selection", nvt_oids=nvts, family=family
            )
            return config_id

        if 0 <= int(chosen_config) < len(config_ids):
            return config_ids[int(chosen_config)]


def get_target(gmp, hosts):
    # create a new target or use an existing
    targets = gmp.get_targets()
    target_ids = targets.xpath("target/@id")

    for i, target in enumerate(targets.xpath("target")):
        name = target.xpath("name/text()")[0]
        print(f"\n({i}) {name}")

    while True:
        if target_ids:
            chosen_target = input(
                "\nChoose your target or create new"
                f" one[0-{len(target_ids) - 1} | n]: "
            )
        else:
            chosen_target = "n"

        if chosen_target == "n":
            if len(hosts) == 0:
                hosts = input("Target hosts (comma separated): ")

            name = input("Name of target: ")

            res = gmp.create_target(name, hosts=hosts.split(","))
            return res.xpath("@id")[0]

        if 0 <= int(chosen_target) < len(target_ids):
            return target_ids[int(chosen_target)]


def get_scanner(gmp):
    res = gmp.get_scanners()
    scanner_ids = res.xpath("scanner/@id")

    for i, scanner in enumerate(res.xpath("scanner")):
        scanner_id = scanner.xpath("@id")[0]
        name = scanner.xpath("name/text()")[0]
        # configs[id] = name
        print(f"\n({i})\n{name}: ({scanner_id})")

    while True:
        chosen_scanner = int(
            input(f"\nChoose your scanner [0-{len(scanner_ids) - 1}]: ")
        )
        if 0 <= chosen_scanner < len(scanner_ids):
            return scanner_ids[chosen_scanner]


def create_and_start_task(
    gmp, task_name, task_comment, config_id, target_id, scanner_id
):
    res = gmp.create_task(
        name=task_name,
        config_id=config_id,
        target_id=target_id,
        scanner_id=scanner_id,
        comment=task_comment,
    )

    # Start the task
    task_id = res.xpath("@id")[0]
    gmp.start_task(task_id)
    print("Task started")


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable

    check_args(args)

    nvt_oid = args.script[1]
    hosts = args.script[2]

    task_name = input("Task name: ")
    task_comment = input("Task comment: ")

    config_id = get_scan_config(gmp, nvt_oid)
    target_id = get_target(gmp, hosts)
    scanner_id = get_scanner(gmp)

    create_and_start_task(
        gmp, task_name, task_comment, config_id, target_id, scanner_id
    )


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# Run with gvm-script --gmp-username admin-user --gmp-password password socket create-tags-from-csv.gmp.py hostname-server tags.csv

import csv
import sys
import time
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from pathlib import Path

from gvm.errors import GvmResponseError
from gvm.protocols.gmp import Gmp
from gvmtools.helper import error_and_exit

HELP_TEXT = (
    "This script pulls tagname, hostnames/IP addresses, and credentials "
    "from a csv file and creates a tag for each row. \n\n"
    "csv file to contain name of tag, ips, and up to 4 credentials previously created \n"
    "name,ip-addresses,credential1,credential2,credential3,credential4"
)


def check_args(args):
    len_args = len(args.script) - 1
    if len_args != 1:
        message = """
        This script pulls tag names from a csv file and creates a tag \
for each row.
        One parameter after the script name is required.

        1. <tags_csvfile>  -- text file containing tagname and hostnames or IP-addresses

        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
ssh --hostname <gsm> scripts/create_tags_from_csv.gmp <tags_csvfile>
        """
        print(message)
        sys.exit()


def parse_args(args: Namespace) -> Namespace:  # pylint: disable=unused-argument
    """Parsing args ..."""

    parser = ArgumentParser(
        prefix_chars="+",
        add_help=False,
        formatter_class=RawTextHelpFormatter,
        description=HELP_TEXT,
    )

    parser.add_argument(
        "+h",
        "++help",
        action="help",
        help="Show this help message and exit.",
    )

    parser.add_argument(
        "tags_csv_file",
        type=str,
        help=("File containing host names / IPs"),
    )

    script_args, _ = parser.parse_known_args(args)
    return script_args


def config_id(
    gmp: Gmp,
    config_name: str,
):
    response_xml = gmp.get_scan_configs(
        filter_string="rows=-1, name= " + config_name
    )
    scan_configs_xml = response_xml.xpath("config")
    config_id = ""

    for scan_config in scan_configs_xml:
        config_id = scan_config.get("id")
    return config_id


def alert_id(
    gmp: Gmp,
    alert_name: str,
):
    response_xml = gmp.get_alerts(filter_string="rows=-1, name=" + alert_name)
    alerts_xml = response_xml.xpath("alert")
    alert_id = ""

    for alert in alerts_xml:
        alert_id = alert.get("id")
    return alert_id


def credential_id(
    gmp: Gmp,
    credName: str,
):
    response_xml = gmp.get_credentials(
        filter_string="rows=-1, name=" + credName
    )
    credentials_xml = response_xml.xpath("credential")
    cred_id = ""

    for credential in credentials_xml:
        cred_id = credential.get("id")
    return cred_id


def target_id(
    gmp: Gmp,
    targetName: str,
):
    response_xml = gmp.get_targets(filter_string="rows=-1, name=" + targetName)
    targets_xml = response_xml.xpath("target")
    target_id = ""

    for target in targets_xml:
        target_id = target.get("id")
    return target_id


def task_id(
    gmp: Gmp,
    taskName: str,
):
    response_xml = gmp.get_tasks(filter_string="rows=-1, name=" + taskName)
    tasks_xml = response_xml.xpath("task")
    task_id = ""

    for task in tasks_xml:
        task_id = task.get("id")
    return task_id


def tag_id(
    gmp: Gmp,
    tagName: str,
):
    response_xml = gmp.get_tags(filter_string="rows=-1, name=" + tagName)
    tags_xml = response_xml.xpath("tag")
    tag_id = ""

    for tag in tags_xml:
        tag_id = tag.get("id")
    return tag_id


def scanner_id(
    gmp: Gmp,
    scanner_name: str,
):
    response_xml = gmp.get_scanners(
        filter_string="rows=-1, name=" + scanner_name
    )
    scanners_xml = response_xml.xpath("scanner")
    scanner_id = ""

    for scanner in scanners_xml:
        scanner_id = scanner.get("id")
    return scanner_id


def schedule_id(
    gmp: Gmp,
    schedule_name: str,
):
    response_xml = gmp.get_schedules(
        filter_string="rows=-1, name=" + schedule_name
    )
    schedules_xml = response_xml.xpath("schedule")
    schedule_id = ""

    for schedule in schedules_xml:
        schedule_id = schedule.get("id")
    return schedule_id


def create_tags(
    gmp: Gmp,
    tag_csv_file: Path,
):
    try:
        numbertags = 0
        with open(tag_csv_file, encoding="utf-8") as csvFile:
            content = csv.reader(csvFile, delimiter=",")  # read the data
            for row in content:  # loop through each row
                if len(row) == 0:
                    continue
                tagType = row[0]
                tagName = row[1]
                tagDescription = row[2]
                tagNameFull = tagName + ":" + tagDescription + ":" + tagType
                if tag_id(gmp, tagNameFull):
                    print(f"Tag: {tagNameFull} already exist")
                    continue
                # Up to ten resources (rows 3 - 12)
                tagResources = []
                if tagType.upper() == "FAIL!":
                    print("Failed!")
                elif tagType.upper() == "ALERT":
                    getUUID = alert_id
                    resource_type = gmp.types.EntityType.ALERT
                elif tagType.upper() == "CONFIG":
                    getUUID = config_id
                    resource_type = gmp.types.EntityType.SCAN_CONFIG
                elif tagType.upper() == "CREDENTIAL":
                    getUUID = credential_id
                    resource_type = gmp.types.EntityType.CREDENTIAL
                elif tagType.upper() == "REPORT":
                    filter = "~" + tagName
                    resource_type = gmp.types.EntityType.REPORT
                elif tagType.upper() == "SCANNER":
                    getUUID = scanner_id
                    resource_type = gmp.types.EntityType.SCANNER
                elif tagType.upper() == "SCHEDULE":
                    getUUID = schedule_id
                    resource_type = gmp.types.EntityType.SCHEDULE
                elif tagType.upper() == "TARGET":
                    getUUID = target_id
                    resource_type = gmp.types.EntityType.TARGET
                elif tagType.upper() == "TASK":
                    getUUID = task_id
                    resource_type = gmp.types.EntityType.TASK
                else:
                    print(
                        "Only alert, config, credential, report, scanner, schedule, target, and task supported"
                    )
                    exit()

                if len(row[3]) >= 1:
                    tagResource = getUUID(gmp, row[3])
                    tagResources.append(tagResource)
                if len(row[4]) >= 1:
                    tagResource = getUUID(gmp, row[4])
                    tagResources.append(tagResource)
                if len(row[5]) >= 1:
                    tagResource = getUUID(gmp, row[5])
                    tagResources.append(tagResource)
                if len(row[6]) >= 1:
                    tagResource = getUUID(gmp, row[6])
                    tagResources.append(tagResource)
                if len(row[7]) >= 1:
                    tagResource = getUUID(gmp, row[7])
                    tagResources.append(tagResource)
                if len(row[8]) >= 1:
                    tagResource = getUUID(gmp, row[8])
                    tagResources.append(tagResource)
                if len(row[9]) >= 1:
                    tagResource = getUUID(gmp, row[9])
                    tagResources.append(tagResource)
                    tagResource = getUUID(gmp, row[10])
                if len(row[10]) >= 1:
                    tagResources.append(tagResource)
                if len(row[11]) >= 1:
                    tagResource = getUUID(gmp, row[11])
                    tagResources.append(tagResource)
                if len(row[12]) >= 1:
                    tagResource = getUUID(gmp, row[12])
                    tagResources.append(tagResource)
                comment = f"Created: {time.strftime('%Y/%m/%d-%H:%M:%S')}"

                if tagType.upper() == "REPORT":
                    try:
                        print("Creating tag: " + tagNameFull)
                        gmp.create_tag(
                            name=tagNameFull,
                            comment=comment,
                            value=tagName,
                            resource_type=resource_type,
                            resource_filter=filter,
                        )
                        numbertags = numbertags + 1
                    except GvmResponseError as gvmerr:
                        print(f"{gvmerr=}, name: {tagNameFull}")
                        pass
                else:
                    try:
                        print("Creating tag: " + tagNameFull)
                        gmp.create_tag(
                            name=tagNameFull,
                            comment=comment,
                            value=tagName,
                            resource_type=resource_type,
                            resource_ids=tagResources,
                        )
                        numbertags = numbertags + 1
                    except GvmResponseError as gvmerr:
                        print(f"{gvmerr=}, name: {tagNameFull}")
                        pass
        csvFile.close()  # close the csv file
    except IOError as e:
        error_and_exit(f"Failed to read tag_csv_file: {str(e)} (exit)")

    if len(row) == 0:
        error_and_exit("tag file is empty (exit)")

    return numbertags


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable
    if args.script:
        args = args.script[1:]

    parsed_args = parse_args(args=args)

    print("Creating tags.\n")

    numbertags = create_tags(
        gmp,
        parsed_args.tags_csv_file,
    )

    numbertags = str(numbertags)
    print("    [" + numbertags + "] tag(s) created!\n")


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2017-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

# ruff: noqa: E501

import sys
import time
from argparse import Namespace

from gvm.protocols.gmp import Gmp
from gvmtools.helper import generate_uuid
from lxml import etree as e


def check_args(args: Namespace) -> None:
    len_args = len(args.script) - 1
    if len_args < 2:
        message = """
        This script will combine desired reports into a single report. \
    The combined report will then be sent to a desired container task. \
    This script will create a container task for the combined report to\
    be sent to, however, if you would like the report to be sent to an \
    existing task, place the report of the desired task first and add  \
    the argument 'first_task'.

        1. <report_1_uuid> --uuid of report to be combined
        2. <report_2_uuid> --uuid of report to be combined
        ...
        n. <report_n_uuid> --uuid of report to be combined

        Example for starting up the routine:
            $ gvm-script --gmp-username=namessh --gmp-password=pass ssh --hostname=hostname \
     scripts/combine-reports.gmp.py \
    "d15a337c-56f3-4208-a462-afeb79eb03b7" \
    "303fa0a6-aa9b-43c4-bac0-66ae0b2d1698" 'first_task'

        """
        print(message)
        sys.exit()


def combine_reports(gmp: Gmp, args: Namespace) -> e.Element:
    new_uuid = generate_uuid()
    combined_report = e.Element(
        "report",
        {
            "id": new_uuid,
            "format_id": "d5da9f67-8551-4e51-807b-b6a873d70e34",
            "extension": "xml",
            "content_type": "text/xml",
        },
    )
    report_elem = e.Element("report", {"id": new_uuid})
    ports_elem = e.Element("ports", {"start": "1", "max": "-1"})
    results_elem = e.Element("results", {"start": "1", "max": "-1"})
    combined_report.append(report_elem)
    report_elem.append(results_elem)

    if "first_task" in args.script:
        arg_len = args.script[1:-1]
    else:
        arg_len = args.script[1:]

    for argument in arg_len:
        current_report = gmp.get_report(
            argument, details=True, ignore_pagination=True
        )[0]
        for port in current_report.xpath("report/ports/port"):
            ports_elem.append(port)
        for result in current_report.xpath("report/results/result"):
            results_elem.append(result)
        for host in current_report.xpath("report/host"):
            report_elem.append(host)

    return combined_report


def send_report(gmp: Gmp, args: Namespace, combined_report: e.Element) -> str:
    if "first_task" in args.script:
        main_report = gmp.get_report(args.script[1])[0]
        task_id = main_report.xpath("//task/@id")[0]
    else:
        the_time = time.strftime("%Y/%m/%d-%H:%M:%S")
        task_id = ""
        task_name = f"Combined_Report_{the_time}"

        res = gmp.create_container_task(
            name=task_name, comment="Created with gvm-tools."
        )

        task_id = res.xpath("//@id")[0]

    combined_report = e.tostring(combined_report)

    res = gmp.import_report(combined_report, task_id=task_id, in_assets=True)

    return res.xpath("//@id")[0]


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable

    check_args(args)

    combined_report = combine_reports(gmp, args)
    send_report(gmp, args, combined_report)


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2024 Martin Boller
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import Namespace

from gvm.protocols.gmp import Gmp
from gvmtools.helper import Table


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=unused-argument

    response_xml = gmp.get_targets(filter_string="rows=-1")
    targets_xml = response_xml.xpath("target")

    heading = [
        "#",
        "Name",
        "Id",
        "Count",
        "SSH Credential",
        "SMB Cred",
        "ESXi Cred",
        "SNMP Cred",
        "Alive test",
    ]

    rows = []
    numberRows = 0

    print("Listing targets.\n")

    for target in targets_xml:
        # Count number of reports
        numberRows = numberRows + 1
        # Cast/convert to text to show in list
        rowNumber = str(numberRows)

        name = "".join(target.xpath("name/text()"))
        maxhosts = "".join(target.xpath("max_hosts/text()"))
        sshcred = "".join(target.xpath("ssh_credential/name/text()"))
        smbcred = "".join(target.xpath("smb_credential/name/text()"))
        esxicred = "".join(target.xpath("esxi_credential/name/text()"))
        snmpcred = "".join(target.xpath("snmp_credential/name/text()"))
        target_id = target.get("id")
        alive_test = "".join(target.xpath("alive_tests/text()"))
        rows.append(
            [
                rowNumber,
                name,
                target_id,
                maxhosts,
                sshcred,
                smbcred,
                esxicred,
                snmpcred,
                alive_test,
            ]
        )

    print(Table(heading=heading, rows=rows))


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

# List vulnerabilities with CERT-BUND IDs and severities from a report,
# per host and CERT-BUND advisory.

import os
import re
import sys
from argparse import ArgumentParser, Namespace
from itertools import zip_longest
from typing import Dict, List, Optional, Sequence, Tuple, TypeVar, overload

from gvm.errors import GvmResponseError
from gvm.protocols.gmp import Gmp

import ssv_csv

# from gvm.xml import pretty_print

sys.path.append(os.path.dirname(args.argv[0]))  # type: ignore


class _Row(Dict, total=False):
    host: str
    port: str
    hostname: str
    name: str
    severity: str
    cves: str
    cb: List[str]


class _Host(Dict, total=False):
    ip: str
    name: str
    operating_system: str


class _CBund(Dict, total=False):
    severity: str
    title: str


@overload
def _get_text(e, other: str) -> str:
    """Signature of _get_text if other is str"""


@overload
def _get_text(e, other: None = None) -> Optional[str]:
    """Signature of _get_text if other is None"""


def _get_text(e, other: Optional[str] = None) -> Optional[str]:
    """Return (recursive) inner text of element if truthy"""
    if e is not None:
        text = "".join(e.itertext())
        text = text.strip()
        if text:
            return text
    return other


@overload
def _assign(
    tgt: _Host, tgtfield: str, src: Dict[str, Optional[str]], srcfield: str
) -> None:
    """Signature of _assign for _Host targets"""


@overload
def _assign(
    tgt: Dict[str, str],
    tgtfield: str,
    src: Dict[str, Optional[str]],
    srcfield: str,
) -> None:
    """Signature of _assign for Dict[str, str] targets"""


def _assign(
    tgt, tgtfield: str, src: Dict[str, Optional[str]], srcfield: str
) -> None:
    """Assign src[srcfield] to tgt[tgtfield] if extant and truthy"""
    if srcfield in src:
        srcval = src[srcfield]
        if srcval:
            tgt[tgtfield] = srcval


def _info(string: str) -> None:
    print("I:", string, file=sys.stderr)


def _warn(string: str) -> None:
    print("W:", string, file=sys.stderr)


def _err(string: str) -> None:
    print("E:", string, file=sys.stderr)


T = TypeVar("T")


def _group_batch(
    tutti: Sequence[T], batch_size: int
) -> Sequence[Tuple[Optional[T], ...]]:
    iterlist = [iter(tutti)] * batch_size
    return list(zip_longest(*iterlist))


# validate CERT-BUND ID
_cb_id_match = re.compile("CB-K[0-9]+/[0-9]+")


# encode for filter_string
def _cb_fmt(cbid: str) -> str:
    if _cb_id_match.fullmatch(cbid) is not None:
        return "uuid=" + cbid
    _warn(f"invalid CERT-BUND ID: {cbid}")
    return ""


def main(gmp: Gmp, args: Namespace) -> None:
    raw_args = [] + args.argv[1:] + args.script_args
    parser = ArgumentParser(
        prog="certbund-report.gmp.py",
        description="Displays CERT-Bund advisories for vulnerabilities.",
        epilog=(
            "Usage: gvm-script [opts] connection_type "
            "certbund-report.gmp.py [Options] ID"
        ),
        add_help=False,
    )
    ogroup = parser.add_argument_group("Options")
    ogroup.add_argument(
        "-H", action="help", help="show this help message and exit"
    )
    ogroup.add_argument(
        "-o",
        "--output",
        metavar="outfile",
        help='write to this CSV file, "-" for stdout (default)',
        default="-",
    )
    ogroup.add_argument(
        "-r",
        "--report",
        action="store_true",
        help="ID is a report ID, not a task ID",
    )
    agroup = parser.add_argument_group("Arguments")
    agroup.add_argument("ID", help="task (or report) ID to analyse")
    script_args = parser.parse_args(raw_args)
    if script_args.report:
        report_id = script_args.ID
    else:
        task_id = script_args.ID
        _info("obtaining task")
        try:
            task = gmp.get_task(task_id)
        except GvmResponseError as e:
            if e.status != "404":
                raise e
            _err(f"task {task_id} not found")
            sys.exit(1)
        try:
            task_report = task.xpath(
                "/get_tasks_response/task[1]/last_report/report[1]"
            )
            report_id = task_report[0].get("id")
        except IndexError:
            _err("task does not have any (finished) report")
            sys.exit(1)
    _info("obtaining report")
    try:
        report = gmp.get_report(report_id, ignore_pagination=True, details=True)
    except GvmResponseError as e:
        if e.status != "404":
            raise e
        _err(f"report {report_id} not found")
        sys.exit(1)
    # with open("report.xml", "w", encoding="utf-8") as rf:
    #    pretty_print(report, file=rf)

    ### gather data
    # + host IP       hosts[row['host']]['ip']
    # - vuln port     row['port']
    # - host name     row.get('hostname',
    #                     hosts[row['host']].get('name', 'N/A'))
    # - host OS       hosts[row['host']].get('os', 'N/A')
    # + vuln name     row['name']
    # + vuln severity row['severity']
    # + vuln CVEs     row['cves']
    # + bund ID       row['cb'] : list(str)
    # + bund severity cbund[…].get('severity', 'N/A')
    # - bund title    cbund[…].get('title', 'N/A')

    orows: List[_Row] = []
    hosts: Dict[str, _Host] = {}
    cbund: Dict[str, _CBund] = {}
    results = report.xpath(
        "/get_reports_response/report/report/results/"
        'result[./nvt/refs/ref/@type="cert-bund"]'
    )
    # pretty_print(results)
    _info(f"processing {len(results)} results")
    for result in results:
        orow: _Row = {}
        r_host = result.find("host")
        asset = r_host.find("asset").attrib["asset_id"]
        hosts[asset] = {"ip": r_host.text}  # more filled in later
        orow["host"] = asset
        r_hostname = _get_text(r_host.find("hostname"))
        if r_hostname:
            orow["hostname"] = r_hostname
        orow["port"] = _get_text(result.find("port"), "N/A")
        orow["name"] = _get_text(result.find("name"), "N/A")
        orow["severity"] = _get_text(result.find("severity"), "N/A")
        r_cve: List[str] = []
        r_cb: List[str] = []
        for ref in result.find("nvt").find("refs").findall("ref"):
            if ref.attrib["type"] == "cve":
                r_cve.append(ref.attrib["id"])
            elif ref.attrib["type"] == "cert-bund":
                cbid = ref.attrib["id"]
                r_cb.append(cbid)
                cbund[cbid] = {}  # more filled in later
        orow["cves"] = ", ".join(r_cve)
        orow["cb"] = r_cb
        orows.append(orow)
    hostdatas = report.xpath("/get_reports_response/report/report/host")
    # pretty_print(hostdatas)
    _info(f"processing {len(hosts)}/{len(hostdatas)} hosts")
    for hostdata in hostdatas:
        asset = hostdata.find("asset").attrib["asset_id"]
        if asset not in hosts:
            continue
        details: Dict[str, Optional[str]] = {}
        details["ip"] = _get_text(hostdata.find("ip"))
        for detail in hostdata.findall("detail"):
            dname = _get_text(detail.find("name"))
            if dname in ("best_os_cpe", "hostname", "OS"):
                details[dname] = _get_text(detail.find("value"))
        hostent: _Host = hosts[asset]
        _assign(hostent, "ip", details, "ip")
        _assign(hostent, "name", details, "hostname")
        # try best_os_cpe first but overwrite with OS if better
        _assign(hostent, "os", details, "best_os_cpe")
        _assign(hostent, "os", details, "OS")

    ### retrieve CERT-BUND Advisories

    _info(f"retrieving {len(cbund)} CERT-BUND advisories")
    # one-by-one
    # cb_retrieve_problem = False
    # for id, cbdata in cbund.items():
    #    try:
    #        cb = gmp.get_cert_bund_advisory(id).find(
    #            'info'
    #        ).find('cert_bund_adv')
    #        cbdata['severity'] = _get_text(cb.find('severity'), 'N/A')
    #        cbdata['title'] = _get_text(cb.find('title'), 'N/A')
    #    except GvmResponseError as e:
    #        if e.status != '404':
    #            raise e
    #        cb_retrieve_problem = True
    # batched
    for cb_batch in _group_batch(list(cbund.keys()), 50):
        actual_batch = [x for x in cb_batch if x is not None]
        fstr = " ".join(map(_cb_fmt, actual_batch)) + " first=1 rows=-1"
        try:
            cbs = gmp.get_cert_bund_advisories(filter_string=fstr)
        except GvmResponseError as e:
            if e.status != "404":
                raise e
            # warn below
            continue
        for cbi in cbs.findall("info"):
            if "id" not in cbi.attrib:
                # we have both <info id="CB-K14/1304"> (which we want)
                # and, for some reason, <info start="1" max="10"/>
                continue
            cbid = cbi.attrib["id"]
            if cbid in cbund:
                cb = cbi.find("cert_bund_adv")
                cbund[cbid]["severity"] = _get_text(cb.find("severity"), "N/A")
                cbund[cbid]["title"] = _get_text(cb.find("title"), "N/A")
    cb_retrieve_problem = {}

    ### output

    _info("emitting CSV")
    if script_args.o == "-":
        outfile = sys.stdout
    else:
        outfile = open(script_args.o, "w", encoding="utf-8")
    writer = ssv_csv.CSVWriter(outfile, sep=",")
    writer.writeln("sep=,")
    writer.write(
        "IP",
        "Port",
        "Hostname",
        "OS",
        "Vulnerability",
        "Severity",
        "CVEs",
        "CertBUND-ID",
        "CertBUND-Severity",
        "CertBUND-Title",
    )
    for row in orows:
        ip = hosts[row["host"]]["ip"]
        port = row["port"]
        hname = row.get("hostname", hosts[row["host"]].get("name", "N/A"))
        operating_system = hosts[row["host"]].get("os", "N/A")
        vname = row["name"]
        vsev = row["severity"]
        cves = row["cves"]
        for cb in row["cb"]:
            if "severity" in cbund[cb]:
                cbsev = cbund[cb]["severity"]
                cbtitle = cbund[cb]["title"]
            else:
                cb_retrieve_problem[cb] = 1
                cbsev = "N/A"
                cbtitle = "N/A (could not be retrieved)"
            writer.write(
                ip,
                port,
                hname,
                operating_system,
                vname,
                vsev,
                cves,
                cb,
                cbsev,
                cbtitle,
            )
    cb_nproblems = len(cb_retrieve_problem)
    if cb_nproblems > 0:
        _warn(f"{cb_nproblems} CERT-BUND advisories could not be obtained")
    else:
        _info("done")


if __name__ == "__gmp__":
    main(gmp, args)  # type: ignore



# SPDX-FileCopyrightText: 2017-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import json
import textwrap
from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from datetime import datetime, timedelta
from pathlib import Path
from random import choice, gauss, randrange, seed

from gvm.protocols.gmp import Gmp
from gvmtools.helper import generate_id, generate_random_ips, generate_uuid
from lxml import etree as e

__version__ = "0.3.1"

HELP_TEXT = f"""
    Random Report Generation Script {__version__} (C) 2017-2023 Greenbone AG

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    This script generates randomized report data.
    """

LOREM_IPSUM = """Lorem ipsum dolor sit amet, consectetur adipiscing elit,
sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.
Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut
aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in
voluptate velit esse cillum dolore eu fugiat nulla pariatur.
Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia
deserunt mollit anim id est laborum."""

local_date_time = datetime.now()


def calculate_timestamp(delta: int = 0) -> str:
    return (
        (local_date_time + timedelta(seconds=delta))
        .astimezone()
        .replace(microsecond=0)
        .isoformat()
    )


def generate_ports(n_ports):
    protocol = ["/tcp", "/udp"]
    return [str(randrange(0, 65536)) + choice(protocol) for i in range(n_ports)]


def generate_report_elem(task, **kwargs):
    rep_format_id = "a994b278-1f62-11e1-96ac-406186ea4fc5"
    rep_id = generate_uuid()
    outer_report_elem = e.Element(
        "report",
        attrib={
            "extension": "xml",
            "id": rep_id,
            "format_id": rep_format_id,
            "content_type": "text/xml",
        },
    )
    owner_elem = e.SubElement(outer_report_elem, "owner")
    e.SubElement(owner_elem, "name").text = "testowner"
    e.SubElement(outer_report_elem, "name").text = "testname"
    e.SubElement(outer_report_elem, "writeable").text = str(0)
    e.SubElement(outer_report_elem, "in_use").text = str(0)
    task_elem = e.SubElement(outer_report_elem, "task", attrib={"id": task[0]})
    e.SubElement(task_elem, "name").text = task[1]
    repform_elem = e.SubElement(
        outer_report_elem, "report_format", attrib={"id": rep_format_id}
    )
    e.SubElement(repform_elem, "name").text = "XML"

    # Generating inner <report> tag
    outer_report_elem.append(generate_inner_report(rep_id, **kwargs))

    return outer_report_elem


def generate_inner_report(
    rep_id, n_results, n_hosts, data, with_descriptions=False, **kwargs
):
    report_elem = e.Element("report", attrib={"id": rep_id})
    results_elem = e.SubElement(
        report_elem, "results", {"max": str(n_results), "start": "1"}
    )

    # Create Hosts, Ports, Data
    hosts = generate_random_ips(n_hosts)  # Host IPs
    ports = generate_ports(n_hosts)
    oid_dict = {host: [] for host in hosts}
    asset_dict = {host: generate_uuid() for host in hosts}
    host_names = {host: generate_id() for host in hosts}
    max_sev = 0.0

    # Create <result> tags with random data
    for _ in range(n_results):
        host_ip = choice(hosts)
        host_port = choice(ports)
        result_elem, oid, severity = generate_result_elem(
            data["vulns"],
            host_ip,
            host_port,
            asset_dict[host_ip],
            host_names[host_ip],
            with_descriptions=with_descriptions,
        )
        if float(severity) > max_sev:
            max_sev = float(severity)

        oid_dict[host_ip].append(oid)
        results_elem.append(result_elem)

    e.SubElement(report_elem, "result_count").text = str(n_results)

    sev_elem = e.Element("severity")
    e.SubElement(sev_elem, "full").text = str(max_sev)
    e.SubElement(sev_elem, "filtered").text = str(max_sev)

    report_elem.append(sev_elem)

    # Create <host> tags with random data
    for host in hosts:
        if len(oid_dict[host]) > 0:
            report_elem.append(
                generate_host_elem(
                    host,
                    oid_dict[host][0],
                    asset_dict[host],
                    host_names[host],
                    data=data,
                    **kwargs,
                )
            )

    return report_elem


def generate_result_elem(
    vulns, host_ip, host_port, host_asset, host_name, with_descriptions=False
):
    result_elem = e.Element("result", {"id": generate_uuid()})

    e.SubElement(result_elem, "name").text = "a_result" + generate_id()
    own = e.SubElement(result_elem, "owner")
    e.SubElement(own, "name").text = generate_id()

    elem = e.Element("modification_time")
    e.SubElement(result_elem, "modification_time").text = calculate_timestamp(0)
    e.SubElement(result_elem, "comment").text = ""
    e.SubElement(result_elem, "creation_time").text = calculate_timestamp(-20)

    host_elem = e.Element("host")
    host_elem.text = host_ip
    e.SubElement(host_elem, "asset", {"asset_id": host_asset}).text = ""
    e.SubElement(host_elem, "hostname").text = host_name
    result_elem.append(host_elem)

    port_elem = e.Element("port")
    port_elem.text = host_port
    result_elem.append(port_elem)

    nvt = vulns[randrange(len(vulns))]
    e.SubElement(result_elem, "severity").text = nvt["severity"]
    nvt_elem = e.Element("nvt", {"oid": nvt["oid"]})
    result_elem.append(nvt_elem)

    if with_descriptions:
        nvt_oid = nvt["oid"]
        description = (
            f"Generated result for VT {nvt_oid} on {host_ip}"
            + f" port {host_port}\n{LOREM_IPSUM}"
        )
        e.SubElement(result_elem, "description").text = description

    e.SubElement(result_elem, "notes").text = "TestNotes"

    result_elem.append(elem)

    return result_elem, nvt["oid"], nvt["severity"]


def generate_host_detail_elem(
    name, value, source_name=None, source_description=None, source_type=None
):
    host_detail_elem = e.Element("detail")
    e.SubElement(host_detail_elem, "name").text = name
    e.SubElement(host_detail_elem, "value").text = value

    if source_name:
        source_elem = e.SubElement(host_detail_elem, "source")
        e.SubElement(source_elem, "name").text = source_name

        if source_description:
            e.SubElement(source_elem, "description").text = source_description

        if source_type:
            e.SubElement(source_elem, "type").text = source_type

    return host_detail_elem


def generate_additional_host_details(
    n_details, host_details, *, not_vuln=False
):
    host_detail_elems = []

    for _ in range(n_details):
        details = None

        if not_vuln:
            details = host_details.copy()
            details["source_name"] += str(randrange(14259, 103585))
        else:
            details = choice(host_details)

        host_detail_elems.append(
            generate_host_detail_elem(
                details["name"],
                details["value"],
                source_name=details.get("source_name"),
                source_description=details.get("source_description"),
                source_type=details.get("source_type"),
            )
        )

    return host_detail_elems


def generate_host_elem(
    host_ip, oid, host_asset, host_name, n_host_details, n_not_vuln, data
):
    host_elem = e.Element("host")
    e.SubElement(host_elem, "ip").text = host_ip
    e.SubElement(host_elem, "asset", {"asset_id": host_asset}).text = ""

    e.SubElement(host_elem, "start").text = calculate_timestamp(-1000)
    e.SubElement(host_elem, "end").text = calculate_timestamp(-30)

    app = choice(list(data["apps"]))
    os = choice(list(data["oss"]))

    host_elem.append(
        generate_host_detail_elem(
            "App",
            data["apps"].get(app),
            source_name=oid,
            source_type="nvt",
        )
    )
    host_elem.append(
        generate_host_detail_elem(
            data["apps"].get(app),
            "/usr/bin/foo",
            source_name=oid,
            source_type="nvt",
        )
    )
    host_elem.append(
        generate_host_detail_elem(
            "hostname",
            host_name,
            source_name=oid,
            source_description="Host Details",
            source_type="nvt",
        )
    )
    host_elem.append(
        generate_host_detail_elem(
            "best_os_txt",
            os,
            source_name=oid,
            source_description="Host Details",
            source_type="nvt",
        )
    )
    host_elem.append(
        generate_host_detail_elem(
            "best_os_cpe",
            data["oss"].get(os),
            source_name=oid,
            source_description="Host Details",
            source_type="nvt",
        )
    )

    if n_host_details:
        host_elem.extend(
            generate_additional_host_details(
                n_host_details, data["host_details"]
            )
        )

    dev = n_not_vuln / 10
    if n_not_vuln:
        host_elem.extend(
            generate_additional_host_details(
                n_not_vuln + randrange(-dev, dev),
                data["not_vuln"],
                not_vuln=True,
            )
        )

    return host_elem


def generate_reports(task, n_reports, n_results, with_gauss, **kwargs):
    reports = []

    if with_gauss:
        n_reports = abs(int(gauss(n_reports, 1)))
        if n_reports == 0:
            n_reports += 1

    for _ in range(n_reports):
        if with_gauss:
            n_results = abs(int(gauss(n_results, 2)))

        report_elem = generate_report_elem(task, n_results=n_results, **kwargs)
        report_elem = e.tostring(report_elem)
        reports.append(report_elem)

    return reports


def generate_target():
    target_name = f"Random_Report_Generation_Target_{calculate_timestamp(0)}"
    target_comment = (
        f"Generated by Random Report Generation Script Version {__version__}."
    )
    target_hosts = ["198.18.0.0/24"]
    target_port_list_id = "c7e03b6c-3bbe-11e1-a057-406186ea4fc5"

    gmp.create_target(
        name=target_name,
        comment=target_comment,
        hosts=target_hosts,
        port_list_id=target_port_list_id,
    )


def generate_data(gmp, n_tasks, task_type, **kwargs):
    if task_type == "scan":
        scanner_id = "08b69003-5fc2-4037-a479-93b440211c73"
        config_id = "085569ce-73ed-11df-83c3-002264764cea"
        generate_target()
        target_id = gmp.get_targets(
            filter_string=f'"Random_Report_Generation_Target_{calculate_timestamp(0)}"'
        ).xpath("//@id")[0]

    for i in range(n_tasks):
        index = f"{{0:0>{len(str(n_tasks))}}}"
        task_name = f"Random_Report_Generation_{task_type.capitalize()}_Task_{calculate_timestamp(0)}_{index.format(i + 1)}"
        task_comment = f"Generated by Random Report Generation Script Version {__version__}."

        gmp.create_container_task(name=task_name, comment=task_comment)

        task_id = gmp.get_tasks(filter_string=f"name={task_name}").xpath(
            "//@id"
        )[0]

        if task_type == "scan":
            gmp.modify_task(task_id=task_id, alterable=1)

        reports = generate_reports(task=(task_id, task_name), **kwargs)

        for report in reports[0:]:
            gmp.import_report(report, task_id=task_id, in_assets=True)

        if task_type == "scan":
            gmp.modify_task(
                task_id=task_id,
                target_id=target_id,
                scanner_id=scanner_id,
                config_id=config_id,
            )


def main(gmp: Gmp, args: Namespace) -> None:
    # pylint: disable=undefined-variable, line-too-long

    parser = ArgumentParser(
        prog="random-report-gen",
        prefix_chars="-",
        description=HELP_TEXT,
        formatter_class=RawTextHelpFormatter,
        add_help=False,
        epilog=textwrap.dedent(
            """
        Example:
            $ gvm-script --gmp-username name --gmp-password pass \
              ssh --hostname <gsm> scripts/gen-random-reports.gmp.py \
              -T 5 -r 4 -R 3 --hosts 10
        """
        ),
    )

    parser.add_argument(
        "-H", action="help", help="Show this help message and exit."
    )

    parser.add_argument(
        "--datafile",
        default=Path(args.script[0]).parent / "default_report_data.json",
        help="A json file containing the following information: "
        "vulnerabilities, operating systems, applications and host details. "
        "Take the default json file as an example.",
    )

    parser.add_argument(
        "--tasks",
        "-T",
        type=int,
        default="1",
        help="Number of Tasks to be generated.",
    )

    parser.add_argument(
        "--reports",
        "-r",
        type=int,
        default="5",
        help="Number of Reports per Task.",
    )

    parser.add_argument(
        "--results",
        "-R",
        type=int,
        default="5",
        help="Number of Results per Report.",
    )

    parser.add_argument(
        "--hosts",
        type=int,
        default="5",
        help="Number of randomized hosts to select from.",
    )

    parser.add_argument(
        "--host-details",
        dest="host_details",
        type=int,
        default="2",
        help="Number of additional host details per host.",
    )

    parser.add_argument(
        "--not-vuln-details",
        dest="not_vuln",
        type=int,
        default="10",
        help="Number of 'NOT_VULN' host details per host.",
    )

    parser.add_argument(
        "--with-descriptions",
        dest="with_descriptions",
        action="store_true",
        help="Adds descriptions to results generated from other result fields"
        " and some dummy text.",
    )

    parser.add_argument(
        "--with-gauss",
        dest="with_gauss",
        action="store_true",
        help="if you would like for the number of reports/task and "
        "results/report to be randomized along a Gaussian distribution.",
    )

    parser.add_argument(
        "--seed", help="RNG Seed, in case the same data should be generated."
    )

    parser.add_argument(
        "--task-type",
        dest="task_type",
        type=str,
        choices=["container", "scan"],
        default="container",
        help="Type of Task(s) to store the generated Reports. Can either "
        "be 'container' or 'scan', default: 'container'.",
    )

    script_args = parser.parse_args(args.script_args)

    if not script_args.seed:
        seed()
    else:
        seed(script_args.seed)

    with open(str(script_args.datafile), encoding="utf-8") as file:
        data = json.load(file)

    print("\n  Generating randomized data...\n")

    generate_data(
        gmp,
        n_tasks=script_args.tasks,
        n_reports=script_args.reports,
        n_results=script_args.results,
        n_hosts=script_args.hosts,
        n_host_details=script_args.host_details,
        n_not_vuln=script_args.not_vuln,
        data=data,
        with_gauss=script_args.with_gauss,
        with_descriptions=script_args.with_descriptions,
        task_type=script_args.task_type,
    )

    print("\n  Generation completed.\n")


if __name__ == "__gmp__":
    main(gmp, args)



# SPDX-FileCopyrightText: 2019-2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import datetime
import sys
from argparse import Namespace

from gvm.protocols.gmp import Gmp


def check_args(args):
    len_args = len(args.script) - 1
    message = """
        This script starts a new scan on the given host.
        It needs one parameters after the script name.

        1. <host_ip>        IP Address of the host system
        2. <port_list_id>   Port List UUID for scanning the host system.
                            Preconfigured UUID might be under
                            /var/lib/gvm/data-objects/gvmd/20.08/port_lists/.
                            ex. iana-tcp-udp is
                            "4a4717fe-57d2-11e1-9a26-406186ea4fc5".

                Example:
            $ gvm-script --gmp-username name --gmp-password pass \
ssh --hostname <gsm> scripts/scan-new-system.gmp.py <host_ip> <port_list_id>
    """
    if len_args != 2:
        print(message)
        sys.exit()


def create_target(gmp, ipaddress, port_list_id):
    # create a unique name by adding the current datetime
    name = f"Suspect Host {ipaddress} {str(datetime.datetime.now())}"

    response = gmp.create_target(
        name=name, hosts=[ipaddress], port_list_id=port_list_id
    )
    return response.get("id")


def create_task(gmp, ipaddress, target_id, scan_config_id, scanner_id):
    name = f"Scan Suspect Host {ipaddress}"
    response = gmp.create_task(
        name=name,
        config_id=scan_config_id,
        target_id=target_id,
        scanner_id=scanner_id,
    )
    return response.get("id")


def start_task(gmp, task_id):
    response = gmp.start_task(task_id)
    # the response is
    # <start_task_response><report_id>id</report_id></start_task_response>
    return response[0].text


def main(gmp: Gmp, args: Namespace) -> None:
    check_args(args)

    ipaddress = args.argv[1]
    port_list_id = args.argv[2]

    target_id = create_target(gmp, ipaddress, port_list_id)

    full_and_fast_scan_config_id = "daba56c8-73ec-11df-a475-002264764cea"
    openvas_scanner_id = "08b69003-5fc2-4037-a479-93b440211c73"
    task_id = create_task(
        gmp,
        ipaddress,
        target_id,
        full_and_fast_scan_config_id,
        openvas_scanner_id,
    )

    report_id = start_task(gmp, task_id)

    print(
        f"Started scan of host {ipaddress}. "
        f"Corresponding report ID is {report_id}"
    )


if __name__ == "__gmp__":
    # pylint: disable=undefined-variable
    main(gmp, args)
